/* Background canvas */
.bg_canvas {
  position: fixed;
  z-index: -1;
  top: 0;
  left: 0;
  right: 0;
  bottom: 0;
  width: 100%;
  height: 100%;
}
.bg_canvas canvas {
  position: absolute;
  z-index: 1;
  top: 0;
  left: 0;
  right: 0;
  bottom: 0;
}

.elementor-section.bg_canvas_covered:not(.elementor-element-edit-mode) {
  background: transparent !important;
}

/*# sourceMappingURL=bg-canvas.css.map */
